/* The following code example is taken from the book
 * "C++ Templates - The Complete Guide"
 * by David Vandevoorde and Nicolai M. Josuttis, Addison-Wesley, 2002
 *
 * (C) Copyright David Vandevoorde and Nicolai M. Josuttis 2002.
 * Permission to copy, use, modify, sell and distribute this software
 * is granted provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied
 * warranty, and with no claim as to its suitability for any purpose.
 */
#include <iostream>
#include <string>
#include <cstdlib>
#include "stack_specialization.h"

int main()
{
    try {
        Stack<int>         intStack;       // stack of ints
        Stack<std::string> stringStack;    // stack of strings

        // manipulate int stack
        intStack.push(7);
        std::cout << intStack.top() << std::endl;
        intStack.pop();

        // manipulate string stack
        stringStack.push("hello");
        std::cout << stringStack.top() << std::endl;
        stringStack.pop();
        stringStack.pop();
    }
    catch (std::exception const& ex)
    {
        std::cerr << "Exception: " << ex.what() << std::endl;
    }

    std::cout << "*************************************************" << std::endl;

    Stack<int>         intStack1;       // stack of ints
    Stack<int>         intStack2;       // stack of ints
    Stack<double>      doubleStack1;    // stack of doubles
    Stack<double>      doubleStack2;    // stack of doubles
    Stack<std::string> stringStack1;    // stack of strings
    Stack<std::string> stringStack2;    // stack of strings

    std::cout << ++intStack1.k << std::endl;
    std::cout << intStack2.k << std::endl;

    std::cout << doubleStack1.k << std::endl;
    std::cout << ++doubleStack2.k << std::endl;
//    std::cout << doubleStack1.k << std::endl;

//    std::cout << ++stringStack1.k << std::endl;
//    std::cout << stringStack2.k << std::endl;

    return 0;

}

